### PETSC library ###################################################################
# The Flow123 requires PETSC library (3.4.x), you have to set 
# the location of PETSC and used PETSC architecture unless you
# would to use system wide settings.
# e.g.

#set(PETSC_DIR "/home/jb/local/petsc-3.4.0")
#set(PETSC_ARCH "gcc_4.4_dbg")

#set(PETSC_DIR "/home/jb/local/petsc-3.4.0")
#set(PETSC_ARCH "gcc_O3_AMD64")

set(PETSC_DIR "/home/jb/local/petsc-3.4.0")
#set(PETSC_ARCH "gcc_4.8_dbg")
set(PETSC_ARCH "gcc_4.8_O3_native_full")


# e.g. Some PETSC installations have missing pthread library. So we use (assuming libpthread.a placed in /usr/lib):
# set( PETSC_ADDITIONAL_LIBS "-lpthread")

# Options for PETSC installer.
#     PETSC_MPI_DIR      - pass MPI to PETSC configure 
#     PETSC_LAPACK_DIR   - pass Lapack to PETSC configure
#     INSTALL_PETSC_ONLY - install only petsc (possibly MPI and BLAS/LAPACK)
#     INSTALL_PETSC_BDDCML - install also: metis, parmetis, hypre, ml, mumps
#     INSTALL_PETSC_OPTIONS - add content of this variable to the PETSC configure command
#
#set(INSTALL_PETSC_ONLY "yes")
# set(INSTALL_PETSC_OPTIONS "--download-hypre=yes")
#set(PETSC_CONFIG "flow123d_mini")

### BDDCML library #############################################################
# BDDCML_ROOT - root directory of BDDCML library
# tested with version 2.2
# If BDDCML_ROOT is set and BDDCML library is not found it is installed automatically.
# If the variable is unset, BDDCML is not used.
# Note: For automatic installation you need MUMPS and ScaLapack, this can be automatically installed with PETSC 
# if you set INSTALL_PETSC_BDDCML.

set(BDDCML_ROOT "_INSTALL_")

### Python setting ###############################################################
# USE_PYTHON - only use python if this is set to "yes"
# We add python library only on explicit request since it leads to tons of errors when debugging with valgrind.
#
set(USE_PYTHON "yes")

### Boost ######################
#set(Boost_FORCE_REBUILD "yes")

### Build setting ####################################################################


### verbose mode ####
# option to produce more verbose compiling
# for the same result you can call make with VERBOSE=1
# e.g.
# make VERBOSE=1 all

set(CMAKE_VERBOSE_MAKEFILE on)


### set compiler ###
# g++ is automatically detected and used
# 
# Clang can not be used without some modifications in Armadillo.
# It is worth to try.
#set( CMAKE_CXX_COMPILER "g++-4.4")
#set(CMAKE_C_COMPILER "gcc-4.4")


### USING CCACHE ###
# If you have installed 'ccache' package, you can enjoy faster builds by setting 
# variable CCACHE_CXX_COMPILER to "ccache __your_favourite_c++_compiler__".
# The helper script is created in the build directory.
#
# ccache works even if you use "make clean-all" command (that's the main advantage).
# Just the first time build with ccache is slower.
# To clean the ccache files, use command: 
# $ ccache -C
# More info and download at: http://ccache.samba.org/
#
# set (CCACHE_CXX_COMPILER "ccache /usr/bin/c++")


### Compiler flags  ####
# There are two predefined configurations: "debug" and "release"
# "debug" set flags: "-g -DDEBUG"
# "release" set flags: "-O3 -DNODEBUG"
#

set(FLOW_BUILD_TYPE release)


# you can override these settings through variable FLOW_CC_FLAGS
#
# Flow recognize  following definitions:
# NODEBUG					- all debug options off
# DEBUG						- all debug options on
# DEBUG_MESSAGES
# DEBUG_ASSERTS
# DEBUG_PROFILER
# DEBUG_FUNCTION_STACK
# HAVE_CXX11				- support for requested C++11 features (constexpr)
#
# e.g. :
#set(FLOW_CC_FLAGS "-g -O0 -DFlow123d_DEBUG -DMPI_PRINTALL")



### External Projects directory ##########################################################
#
# The directory where possible external projects should be stored and built.
# The default location is ${PROJECT_SOURCE_DIR}/external_projects.

#set(EXTERNAL_PROJECT_DIR "") 
